function mu=eigTMN(myDDE,par,s,h,M,N)
%EIGTMN solution operator approach for linear NONAUTONOMOUS DDEs.
%  mu=EIGTMN(myDDE,par,s,h,M,N) approximates the eigenvalues mu of the
%  evolution operator T(s+h,s) associated to the system of d linear and
%  NONAUTONOMOUS Delay Differential Equations (DDEs) defined in the script
%  "myDDE.m". It uses pseudospetral collocation methods to discretize the
%  operator as described in [2, Chapter 6] and originally in [3],
%  according to the notation in [2]. par is a vector of possible model 
%  parameters, see "myDDE.m"; M and N are discretization indices, see [2].
%
%  Example of call: mu=eigTMN('myDDE',par,s,h,M,N);
%
%  Version 2.0, february 2, 2016.
%
%  References:
%  [1] J.P. Berrut and L.N. Trefethen, "Barycentric Lagrange
%      interpolation", SIAM Rev. 46(3):501-517, 2004.
%  [2] D. Breda, S. Maset and R. Vermiglio, "Stability of linear delay
%      differential equations - A numerical approach with MATLAB", in
%      Control, Automation and Robotics, T. Basar, A. Bicchi and M. Krstic
%      eds., Springer, New York, 2015, ISBN 978-1-4939-2106-5.
%  [3] D. Breda, S. Maset and R. Vermiglio, "Approximation of eigenvalues
%      of evolution operators for linear retarded functional differential
%      equations", SIAM J. Numer. Anal. 50(3):1456-1483, 2012.
%  [4] L.N. Trefethen, "Spectral methods in Matlab", SIAM, 2000.

%% DDE MODEL CONVERSION
[d,tau,p,A,B,C,flagC]=conversion(myDDE,par);
Id=eye(d);
TAU=max(tau);
                                              
%% MESH OmegaM IN [-TAU,0] (ordered from right=0 to left=-TAU)
zM=cos((0:M)*pi/M);                    %Chebyshev II nodes in [-1,1]
[wbM]=barywei(zM);                     %Chebyshev II barycentric weights in [-1,1]
[wqM]=quadwei(M);                      %Chebyshev II quadrature weights in [-1,1]
if h>=TAU                              %CASE h>=TAU
    OmegaM=TAU*(zM-1)/2;               %nodes in [-TAU,0]
else                                   %CASE h<TAU
    Q=ceil(TAU/h);                     %Q=min q s.t. qh>=TAU
    OmegaM=zeros(Q,M+1);               %OmegaM(q,m+1)=mth node in [-qh,-(q-1)h]
    for q=1:Q-1                        %
        OmegaM(q,:)=h*((zM+1)/2-q);    %qth row: nodes in [-qh,-(q-1)h]
    end                                %
    OmegaM(Q,:)=(TAU-(Q-1)*h)*zM/2-... %Qth row: nodes in [-TAU,-(Q-1)h]
        (TAU+(Q-1)*h)/2;               %
end                                    %

%% MESH OmegaN IN [0,h] (ordered from left=0 to right=h)
zN=cos((2*(1:N)-1)*pi/(2*N)); %Chebyshev I nodes in [-1,1]
wbN=barywei(zN);              %Chebyshev I barycentric weights in [-1,1]
OmegaN=h*(1-zN)/2;            %nodes in [0,h]

%% MATRIX TM1
if h>=TAU                                   %CASE h>=TAU
    TM=[ones(M+1,1),zeros(M+1,M)];          %
else                                        %CASE h<TAU
    TM=kron(diag(ones(Q-1,1),-1),eye(M));   %first QMxQM entries
    TM(Q*M+1,Q*M+1)=0;                      %enlarge to (QM+1)x(QM+1)
    TM(1:M,1)=ones(M,1);                    %add 1's to 1st column of block(1,1)
    a=OmegaM(Q-1,M+1);                      %update block(Q,Q-1) by evaluating
    b=OmegaM(Q-1,1);                        %Lagrange polynomials through the
    for m=0:M                               %1st barycentric formula [1]
        z=(2*(h+OmegaM(Q,m+1))-a-b)/(b-a);  %point in [-1,1]
        nodal=prod(z-zM);                   %nodal polynomial in z
        for j=0:M                           %...
            TM((Q-1)*M+m+1,(Q-2)*M+j+1)=... %1st barycentric formula [1]
                nodal*wbM(j+1)/(z-zM(j+1)); %for the jth Lagrange polynomial
        end                                 %...
    end                                     %
    TM(isnan(TM))=1;                        %remove singularities at z=zM
end                                         %
TM1=kron(TM,Id);                            %block structure for systems

%% MATRIX TMN2
if h>=TAU                                     %CASE h>=TAU
    TMN=zeros(M+1,N);                         %initialization
    for m=0:M                                 %along rows
        b=h+OmegaM(1,m+1);                    %integral upper extremum
        for n=1:N                             %along columns
            int=0;                            %initialize quadrature sum
            for j=0:M                         %quadrature addends
                z=1-b*(zM(j+1)+1)/h;          %point in [-1,1]
                lagvaln=prod(z-zN)*wbN(n)/... %1st barycentric formula [1]
                    (z-zN(n));                %for the nth Lagrange polynomial
                lagvaln(isnan(lagvaln))=1;    %remove singularities at z=zN
                int=int+wqM(j+1)*lagvaln;     %quadrature increment
            end                               %
            TMN(m+1,n)=b/2*int;               %final quadrature
        end                                   %
    end                                       %
else                                          %CASE h<TAU
    TMN=zeros(Q*M+1,N);                       %initialization
    for m=0:M-1                               %along rows
        b=h+OmegaM(1,m+1);                    %integral upper extremum
        for n=1:N                             %along columns
            int=0;                            %initialize quadrature sum
            for j=0:M                         %quadrature addends                      
                z=1-b*(zM(j+1)+1)/h;          %point in [-1,1]
                lagvaln=prod(z-zN)*wbN(n)/... %1st barycentric formula [1]
                    (z-zN(n));                %for the nth Lagrange polynomial
                lagvaln(isnan(lagvaln))=1;    %remove singularities at z=zN
                int=int+wqM(j+1)*lagvaln;     %quadrature increment
            end                               %
            TMN(m+1,n)=b/2*int;               %final quadrature
        end                                   %
    end                                       %
end                                           %
TMN2=kron(TMN,Id);                            %block structure for systems

%% MATRIX UMN1
if h>=TAU                                      %CASE h>=TAU
    UMN1=zeros(d*N,d*(M+1));                   %initialization
    n=1;                                       %Nhat=max n s.t. OmegaN(n)<TAU
    while (n<=N)&&(OmegaN(n)<TAU)              %or 0 if OmegaN(n)>=TAU for all n
        n=n+1;                                 %
    end                                        %
    Nhat=n-1;                                  %
    for n=1:Nhat                               %along rows for n=1,...,Nhat
        tn=OmegaN(n);                          %current point
        ind=find(tn>=tau,1,'last')-1;          %ind=max k s.t. tau_{k}<=tn<tau_{k+1}
        for m=0:M                              %along columns
            for k=ind+1:p                      %discrete delay terms in E_nm
                Bk=B{k};                       %discrete delay matrix
                z=2*(tn-tau(k+1))/TAU+1;       %point in [-1,1]
                lagvalm=prod(z-zM)*...         %1st barycentric formula [1]
                    wbM(m+1)/(z-zM(m+1));      %for the mth Lagrange polynomial
                lagvalm(isnan(lagvalm))=1;     %remove singularities at z=zM
                UMN1(d*(n-1)+1:d*n,d*m+1:d*(m+1))=...
                    UMN1(d*(n-1)+1:d*n,d*m+1:d*(m+1))+...
                    lagvalm*Bk(s+tn,d,par);
            end                                %
        end                                    %
        UMN1(d*(n-1)+1:d*n,1:d)=...            %current time term in D_n0
            UMN1(d*(n-1)+1:d*n,1:d)+A(s+tn,d,par);
        for k=1:ind                            %discrete delay terms in D_n0
            Bk=B{k};                           %discrete delay matrix
            UMN1(d*(n-1)+1:d*n,1:d)=UMN1(d*(n-1)+1:d*n,1:d)+Bk(s+tn,d,par);
        end                                    %
    end                                        %
    for n=Nhat+1:N                             %along rows for n=Nhat+1,...,N
        tn=OmegaN(n);                          %current point
        UMN1(d*(n-1)+1:d*n,1:d)=...            %current time term in D_n0
            UMN1(d*(n-1)+1:d*n,1:d)+A(s+tn,d,par);
        for k=1:p                              %discrete delay terms in D_n0
            Bk=B{k};                           %discrete delay matrix
            UMN1(d*(n-1)+1:d*n,1:d)=UMN1(d*(n-1)+1:d*n,1:d)+Bk(s+tn,d,par);
        end                                    %
    end                                        %
    if flagC==1                                    %distributed delays
        for n=1:Nhat                               %along rows for n=1,...,Nhat
            tn=OmegaN(n);                          %current point
            ind=find(tn>=tau,1,'last')-1;          %ind=max k s.t. tau_{k}<=tn<tau_{k+1}
            for m=0:M                              %along columns
                INT=0;                             %initialize piecewise quadrature for
                                                   %distributed delay terms in E_nm
                int=0;                             %initialize quadrature on
                                                   %[-tau_{ind+1},-tn]
                Ci=C{ind+1};                       %distributed delay kernel
                for j=0:M                          %quadrature addends
                    theta=-((tau(ind+2)-tn)*zM(j+1)+...
                        tau(ind+2)+tn)/2;          %point in [-tau_{ind+1},-tn]
                    z=2*(tn+theta)/TAU+1;          %point in [-1,1]
                    lagvalm=prod(z-zM)*...         %1st barycentric formula [1]
                        wbM(m+1)/(z-zM(m+1));      %for the mth Lagrange polynomial
                    lagvalm(isnan(lagvalm))=1;     %remove singularities at z=zM
                    int=int+wqM(j+1)*lagvalm*...
                        Ci(s+tn,theta,d,par);      %quadrature increment
                end                                %
                INT=(tau(ind+2)-tn)/2*int;         %add to piecewise quadrature
                for k=ind+2:p                      %
                    int=0;                         %initialize quadrature on
                                                   %[-tau_{k},-tau_{k-1}]
                    Ck=C{k};                       %distributed delay kernel
                    for j=0:M                      %quadrature addends
                        theta=-((tau(k+1)-tau(k))*zM(j+1)+...
                            (tau(k+1)+tau(k)))/2;  %point in [-tau_{k},-tau_{k-1}]
                        z=2*(tn+theta)/TAU+1;      %point in [-1,1]
                        lagvalm=prod(z-zM)*...     %1st barycentric formula [1]
                            wbM(m+1)/(z-zM(m+1));  %for the mth Lagrange polynomial
                        lagvalm(isnan(lagvalm))=1; %remove singularities at z=zM
                        int=int+wqM(j+1)*lagvalm*...
                            Ck(s+tn,theta,d,par);  %quadrature increment
                    end                            %
                    INT=INT+...                    %
                        (tau(k+1)-tau(k))/2*int;   %add to piecewise quadrature
                end                                %
                UMN1(d*(n-1)+1:d*n,d*m+1:d*(m+1))=...
                    UMN1(d*(n-1)+1:d*n,d*m+1:d*(m+1))+...
                        INT;                       %update
            end                                    %
            INT=0;                                 %initialize piecewise quadrature for
                                                   %distributed delay terms in D_n0
            for k=1:ind                            %
                int=0;                             %initialize quadrature on
                                                   %[-tau_{k},-tau_{k-1}]
                Ck=C{k};                           %distributed delay kernel
                for j=0:M                          %quadrature addends
                    int=int+wqM(j+1)*Ck(s+tn,-((tau(k+1)-tau(k))*zM(j+1)+...
                        tau(k+1)+tau(k))/2,d,par); %quadrature increment
                end                                %
                INT=INT+(tau(k+1)-tau(k))/2*int;   %add to piecewise quadrature
            end                                    %
            if ind<p                               %
                int=0;                             %initialize quadrature on
                                                   %[-tn,-tau_{ind}]
                Ci=C{ind+1};                       %distributed delay kernel
                for j=0:M                          %quadrature addends
                    int=int+wqM(j+1)*Ci(s+tn,-((tn-tau(ind+1))*zM(j+1)+tn+...
                        tau(ind+1))/2,d,par);      %quadrature increment
                end                                %
                INT=INT+(tn-tau(ind+1))/2*int;     %add to piecewise quadrature
            end                                    %
            UMN1(d*(n-1)+1:d*n,1:d)=...
                UMN1(d*(n-1)+1:d*n,1:d)+INT;       %update
        end                                        %
        for n=Nhat+1:N                             %along rows for n=Nhat+1,...,N
            tn=OmegaN(n);                          %current point
            INT=0;                                 %initialize piecewise quadrature for
                                                   %distributed delay terms in D_n0
            for k=1:p                              %
                int=0;                             %initialize quadrature on
                                                   %[-tau_{k},-tau_{k-1}]
                Ck=C{k};                           %distributed delay kernel
                for j=0:M                          %quadrature addends
                    int=int+wqM(j+1)*Ck(s+tn,-((tau(k+1)-tau(k))*zM(j+1)+...
                        tau(k+1)+tau(k))/2,d,par); %quadrature increment
                end                                %
                INT=INT+(tau(k+1)-tau(k))/2*int;   %add to piecewise quadrature
            end                                    %
            UMN1(d*(n-1)+1:d*n,1:d)=...
                UMN1(d*(n-1)+1:d*n,1:d)+INT;       %update
        end                                        %
    end                                            %
else                                       %CASE h<TAU
    UMN1=zeros(d*N,d*(Q*M+1));             %initialization
    n=1;                                   %Nhat=max n s.t. OmegaN(n)<TAU-(Q-1)h
    while (n<=N)&&(OmegaN(n)<TAU-(Q-1)*h)  %or 0 if OmegaN(n)>=TAU-(Q-1)h for
        n=n+1;                             %all n
    end                                    %
    Nhat=n-1;                              %
    for n=1:N                              %along rows
        tn=OmegaN(n);                      %current point
        if n<=Nhat                         %definition of qn
            qn=Q;                          %
        else qn=Q-1;                       %
        end                                %
        if qn-1>=1                         %definition of tn1
            tn1=tn+h;                      %
        else tn1=TAU;                      %
        end                                %
        ind=find(tn>=tau,1,'last')-1;      %ind=max k s.t. tau_{k}<=tn<tau_{k+1}
        ind1=find(tn1>=tau,1,'last')-1;    %ind1=max k s.t. tau_{k}<=tn1<tau_{k+1}
        UMN1(d*(n-1)+1:d*n,1:d)=...        %current time term in F_n10
            UMN1(d*(n-1)+1:d*n,1:d)+...
            A(s+tn,d,par);
        for k=1:ind                        %1st discrete delay terms in F_n10
            Bk=B{k};                       %discrete delay matrix
            UMN1(d*(n-1)+1:d*n,1:d)=UMN1(d*(n-1)+1:d*n,1:d)+Bk(s+tn,d,par);
        end                                %
        for k=ind+1:ind1                   %2nd discrete delay terms in F_n10
            Bk=B{k};                       %discrete delay matrix
            z=2*(tn-tau(k+1))/h+1;         %point in [-1,1]
            lagvalm=prod(z-zM)*...         %1st barycentric formula [1]
                wbM(m+1)/(z-zM(m+1));      %for the mth Lagrange polynomial
            lagvalm(isnan(lagvalm))=1;     %remove singularities at z=zM
            UMN1(d*(n-1)+1:d*n,1:d)=UMN1(d*(n-1)+1:d*n,1:d)+...
            lagvalm*Bk(s+tn,d,par);
        end                                %
        if flagC==1                                %distributed delays
            INT=0;                                 %initialize piecewise quadrature
                                                   %for 1st distributed delay
                                                   %terms in F_n10
            for k=1:ind                            %
                int=0;                             %initialize quadrature on
                                                   %[-tau_{k-1},-tau_{k}]
                Ck=C{k};                           %distributed delay kernel
                for j=0:M                          %quadrature addends
                    int=int+wqM(j+1)*Ck(s+tn,-...  %quadrature increment
                        ((tau(k+1)-tau(k))*zM(j+1)+tau(k+1)+tau(k))/2,d,par);
                end                                %
                INT=INT+(tau(k+1)-tau(k))/2*int;   %add to piecewise quadrature
            end                                    %
            if ind<p
                int=0;                             %initialize quadrature on
                                                   %[-tn,-tau_{ind}]
                Ci=C{ind+1};                       %distributed delay kernel
                for j=0:M                          %quadrature addends
                    int=int+wqM(j+1)*Ci(s+tn,-...  %quadrature increment
                        ((tn-tau(ind+1))*zM(j+1)+tn+tau(ind+1))/2,d,par);
                end                                %
                INT=INT+(tn-tau(ind+1))/2*int;     %add to piecewise quadrature
            end                                    %
            UMN1(d*(n-1)+1:d*n,1:d)=...
                UMN1(d*(n-1)+1:d*n,1:d)+INT;       %update
            INT=0;                                 %initialize piecewise quadrature
                                                   %for 2nd distributed delay
                                                   %terms in F_n10
            if ind==ind1                           %
                int=0;                             %initialize quadrature on
                                                   %[-tn1,-tn]
                Ci=C{ind+1};                       %distributed delay kernel
                for j=0:M                          %quadrature addends
                    theta=-((tn1-tn)*...           %point in [-tn1,-tn]
                        zM(j+1)+tn1+tn)/2; 
                    z=2*(tn+theta)/h+1;            %point in [-1,1]
                    lagvalm=prod(z-zM)*...         %1st barycentric formula [1]
                        wbM(1)/(z-zM(1));          %for the 1st Lagrange polynomial
                    lagvalm(isnan(lagvalm))=1;     %remove singularities at z=zM
                    int=int+wqM(j+1)*...           %quadrature increment
                        lagvalm*Ci(s+tn,theta,d,par);
                end                                %
                INT=INT+(tn1-tn)/2*int;            %add to piecewise quadrature
            else                                   %
                int=0;                             %initialize quadrature on
                                                   %[-tau_{ind+1},-tn]
                Ci=C{ind+1};                       %distributed delay kernel
                for j=0:M                          %quadrature addends
                    theta=-((tau(ind+2)-tn)*...    %point in [-tau_{ind+1},-tn]
                        zM(j+1)+tau(ind+2)+tn)/2;
                    z=2*(tn+theta)/h+1;            %point in [-1,1]
                    lagvalm=prod(z-zM)*...         %1st barycentric formula [1]
                        wbM(1)/(z-zM(1));          %for the 1st Lagrange polynomial
                    lagvalm(isnan(lagvalm))=1;     %remove singularities at z=zM
                    int=int+wqM(j+1)*...           %quadrature increment
                        lagvalm*Ci(s+tn,theta,d,par);
                end                                %
                INT=INT+(tau(ind+2)-tn)/2*int;     %add to piecewise quadrature
                for k=ind+2:ind1                   %
                    int=0;                         %initialize quadrature on
                                                   %[-tau_{k-1},-tau_{k}]
                    Ck=C{k};                       %distributed delay kernel
                    for j=0:M                      %quadrature addends
                        theta=-((tau(k+1)-...      %point in [-tau_{k-1},-tau_{k}]
                            tau(k))*zM(j+1)+tau(k+1)+tau(k))/2;
                        z=2*(tn+theta)/h+1;        %point in [-1,1]
                        lagvalm=prod(z-zM)*...     %1st barycentric formula [1]
                            wbM(1)/(z-zM(1));      %for the 1st Lagrange polynomial
                        lagvalm(isnan(lagvalm))=1; %remove singularities at z=zM
                        int=int+wqM(j+1)*...       %quadrature increment
                            lagvalm*Ck(s+tn,theta,d,par);
                    end                            %
                    INT=INT+...
                        (tau(k+1)-tau(k))/2*int;   %add to piecewise quadrature
                end                                %
                if ind1<p                          %
                    int=0;                         %initialize quadrature on
                                                   %[-tn1,-tau_{ind1}]
                    Ci1=C{ind1+1};                 %distributed delay kernel
                    for j=0:M                      %quadrature addends
                        theta=-((tn1-...           %point in [-tn1,-tau_{ind1}]
                            tau(ind1+1))*zM(j+1)+tn1+tau(ind1+1))/2;
                        z=2*(tn+theta)/h+1;        %point in [-1,1]
                        lagvalm=prod(z-zM)*...     %1st barycentric formula [1]
                            wbM(1)/(z-zM(1));      %for the 1st Lagrange polynomial
                        lagvalm(isnan(lagvalm))=1; %remove singularities at z=zM
                        int=int+wqM(j+1)*...       %quadrature increment
                            lagvalm*Ci1(s+tn,theta,d,par);
                    end                            %
                    INT=INT+...
                        (tn1-tau(ind1+1))/2*int;   %add to piecewise quadrature
                end                                %
            end                                    %
            UMN1(d*(n-1)+1:d*n,1:d)=...
                UMN1(d*(n-1)+1:d*n,1:d)+INT;       %update
        end                                        %
        for q=2:qn                                 %along block columns
            if qn-1>=q                             %definition of tnq
                tnq=tn+q*h;                        %
            else tnq=TAU;                          %
            end                                    %
            tnq1=tn+(q-1)*h;                       %definition of tnq1
            tnq2=tnq1-h;                           %definition of tnq2
            ind=find(tnq>=tau,1,'last')-1;         %ind=max k s.t. tau_{k}<=tnq<tau_{k+1}
            ind1=find(tnq1>=tau,1,'last')-1;       %ind1=max k s.t. tau_{k}<=tnq1<tau_{k+1}
            ind2=find(tnq2>=tau,1,'last')-1;       %ind2=max k s.t. tau_{k}<=tnq2<tau_{k+1}
            for k=ind2+1:ind1                      %1st discrete delay terms in F_nq0
                Bk=B{k};                           %discrete delay matrix
                z=2*(tn-tau(k+1))/h+2*q-3;         %point in [-1,1]
                lagvalM=prod(z-zM)*...             %1st barycentric formula [1]
                    wbM(M+1)/(z-zM(M+1));          %for the mth Lagrange polynomial
                lagvalM(isnan(lagvalM))=1;         %remove singularities at z=zM
                UMN1(d*(n-1)+1:d*n,(q-1)*M*d+1:((q-1)*M+1)*d)=...
                    UMN1(d*(n-1)+1:d*n,(q-1)*M*d+1:((q-1)*M+1)*d)+...
                    lagvalM*Bk(s+tn,d,par);
            end                                    %
            for k=ind1+1:ind                       %2nd discrete delay terms in F_nq0
                Bk=B{k};                           %discrete delay matrix
                if  qn-1>=q                        %point in [-1,1]
                    z=2*(tn-tau(k+1))/h+2*q-1;     %
                elseif n<=Nhat                     %
                    z=(2*(tn-tau(k+1))+...         %
                        TAU+(q-1)*h)/(-(q-1)*h+TAU);
                else z=2*(tn-tau(k+1))/h+2*Q-3;    %
                end                                %
                lagval0=prod(z-zM)*...             %1st barycentric formula [1]
                    wbM(1)/(z-zM(1));              %for the mth Lagrange polynomial
                lagval0(isnan(lagval0))=1;         %remove singularities at z=zM
                UMN1(d*(n-1)+1:d*n,(q-1)*M*d+1:((q-1)*M+1)*d)=...
                    UMN1(d*(n-1)+1:d*n,(q-1)*M*d+1:((q-1)*M+1)*d)+...
                    lagval0*Bk(s+tn,d,par);
            end                                    %
            if flagC==1                            %distributed delays
                INT=0;                             %initialize piecewise quadrature
                                                   %for 1st distributed delay
                                                   %terms in F_nq0
                if ind1==ind2                      %
                    int=0;                         %initialize quadrature on
                                                   %[-tnq1,-tnq2]
                    Ci2=C{ind2+1};                 %distributed delay kernel
                    for j=0:M                      %quadrature addends
                        theta=-((tnq1-tnq2)*zM(j+1)+...
                            tnq2+tnq1)/2;          %point in [-tnq1,-tnq2]
                        z=2*(tn+theta)/h+2*q-3;    %point in [-1,1]
                        lagvalm=prod(z-zM)*...     %1st barycentric formula [1]
                            wbM(M+1)/(z-zM(M+1));  %for the Mth Lagrange polynomial
                        lagvalm(isnan(lagvalm))=1; %remove singularites at z=zM
                        int=int+wqM(j+1)*...       %quadrature increment
                            lagvalm*Ci2(s+tn,theta,d,par);
                    end                            %
                    INT=INT+(tnq1-tnq2)/2*int;     %add to piecewise quadrature
                else                               %
                    int=0;                         %initialize quadrature on
                                                   %[-tau_{ind2+1},-tnq2]
                    Ci2=C{ind2+1};                 %distributed delay kernel
                    for j=0:M                      %quadrature addends
                        theta=-((tau(ind2+2)-tnq2)*zM(j+1)+...
                            tau(ind2+2)+tnq2)/2;   %point in [-tau_{ind2+1},-tnq2]
                        z=2*(tn+theta)/h+2*q-3;    %point in [-1,1]
                        lagvalm=prod(z-zM)*...     %1st barycentric formula [1]
                            wbM(M+1)/(z-zM(M+1));  %for the Mth Lagrange polynomial
                        lagvalm(isnan(lagvalm))=1; %remove singularites at z=zM
                        int=int+wqM(j+1)*...       %quadrature increment
                            lagvalm*Ci2(s+tn,theta,d,par);
                    end                            %
                    INT=INT+...
                        (tau(ind2+2)-tnq2)/2*int;  %add to piecewise quadrature
                    for k=ind2+2:ind1                  %
                        int=0;                         %initialize quadrature on
                                                       %[-tau_{k-1},-tau_{k}]
                        Ck=C{k};                       %distributed delay kernel
                        for j=0:M                      %quadrature addends
                            theta=-((tau(k+1)-tau(k))*zM(j+1)+...
                                tau(k+1)+tau(k))/2;    %point in [-tau_{k-1},-tau_{k}]
                            z=2*(tn+theta)/h+2*q-3;    %point in [-1,1]
                            lagvalm=prod(z-zM)*...     %1st barycentric formula [1]
                                wbM(M+1)/(z-zM(M+1));  %for the Mth Lagrange polynomial
                            lagvalm(isnan(lagvalm))=1; %remove singularites at z=zM
                            int=int+wqM(j+1)*...       %quadrature increment
                                lagvalm*Ck(s+tn,theta,d,par);
                        end                            %
                        INT=INT+...
                            (tau(k+1)-tau(k))/2*int;   %add to piecewise quadrature
                    end                                %
                    if ind1<p                          %
                        int=0;                         %initialize quadrature on
                                                       %[-tnq1,-tau_{ind1}]
                        Ci1=C{ind1+1};                 %distributed delay kernel
                        for j=0:M                      %quadrature addends
                            theta=-((tnq1-tau(ind1+1))*zM(j+1)+...
                                tau(ind1+1)+tnq1)/2;   %point in [-tnq1,-tau_{ind1}]
                            z=2*(tn+theta)/h+2*q-3;    %point in [-1,1]
                            lagvalm=prod(z-zM)*...     %1st barycentric formula [1]
                                wbM(M+1)/(z-zM(M+1));  %for the Mth Lagrange polynomial
                            lagvalm(isnan(lagvalm))=1; %remove singularites at z=zM
                            int=int+wqM(j+1)*...       %quadrature increment
                                lagvalm*Ci1(s+tn,theta,d,par);
                        end                            %
                        INT=INT+...
                            (tnq1-tau(ind1+1))/2*int;  %add to piecewise quadrature
                    end                                %
                end                                    %
                UMN1(d*(n-1)+1:d*n,(q-1)*M*d+1:((q-1)*M+1)*d)=...
                        UMN1(d*(n-1)+1:d*n,(q-1)*M*d+1:((q-1)*M+1)*d)+...
                        INT;                           %update
                INT=0;                                 %initialize piecewise quadrature
                                                       %for 2nd distributed
                                                       %delay terms in F_nq0
                if ind==ind1                           %
                    int=0;                             %initialize quadrature on
                                                       %[-tnq,-tnq1]
                    Ci=C{ind+1};                       %distributed delay kernel
                    for j=0:M                          %quadrature addends
                        theta=-((tnq-tnq1)*zM(j+1)+...
                            tnq1+tnq)/2;               %point in [-tnq,-tnq1]
                        if  qn-1>=q                    %point in [-1,1]
                            z=2*(tn+theta)/h+2*q-1;    %
                        elseif n<=Nhat                 %
                            z=(2*(tn+theta)+...        %
                                TAU+(q-1)*h)/(-(q-1)*h+TAU);
                        else z=2*(tn+theta)/h+2*Q-3;   %
                        end                            %
                        lagvalm=prod(z-zM)*...         %1st barycentric formula [1]
                            wbM(1)/(z-zM(1));          %for the 1st Lagrange polynomial
                        lagvalm(isnan(lagvalm))=1;     %remove singularities at z=zM
                        int=int+wqM(j+1)*...           %quadrature increment
                            lagvalm*Ci(s+tn,theta,d,par);
                    end                                %
                    INT=INT+(tnq-tnq1)/2*int;          %add to piecewise quadrature
                else                                   %
                    int=0;                             %initialize quadrature on
                                                       %[-tau_{ind1+1},-tnq1]
                    Ci1=C{ind1+1};                     %distributed delay kernel
                    for j=0:M                          %quadrature addends
                        theta=-((tau(ind1+2)-tnq1)*zM(j+1)+...
                            tau(ind1+2)+tnq1)/2;       %point in [-tau_{ind1+1},-tnq1]
                        if  qn-1>=q                    %point in [-1,1]
                            z=2*(tn+theta)/h+2*q-1;    %
                        elseif n<=Nhat                 %
                            z=(2*(tn+theta)+...        %
                                TAU+(q-1)*h)/(-(q-1)*h+TAU);
                        else z=2*(tn+theta)/h+2*Q-3;   %
                        end                            %
                        lagvalm=prod(z-zM)*...         %1st barycentric formula [1]
                            wbM(1)/(z-zM(1));          %for the 1st Lagrange polynomial
                        lagvalm(isnan(lagvalm))=1;     %remove singularities at z=zM
                        int=int+wqM(j+1)*...           %quadrature increment
                            lagvalm*Ci1(s+tn,theta,d,par);
                    end                                %
                    INT=INT+(tau(ind1+2)-tnq1)/2*int;  %add to piecewise quadrature
                    for k=ind1+2:ind                   %
                        int=0;                         %initialize quadrature on
                                                       %[-tau_{k-1},-tau_{k}]
                        Ck=C{k};                       %distributed delay kernel
                        for j=0:M                      %quadrature addends
                            theta=-((tau(k+1)-tau(k))*zM(j+1)+...
                                tau(k+1)+tau(k))/2;    %point in [-tau_{k-1},-tau_{k}]
                            if  qn-1>=q                %point in [-1,1]
                                z=2*(tn+theta)/h...    %
                                    +2*q-1;            %
                            elseif n<=Nhat             %
                                z=(2*(tn+theta)+...    %
                                    TAU+(q-1)*h)/(-(q-1)*h+TAU);
                            else z=2*(tn+theta)/h+...  %
                                    2*Q-3;             %
                            end                        %
                            lagvalm=prod(z-zM)*...     %1st barycentric formula [1]
                                wbM(1)/(z-zM(1));      %for the 1st Lagrange polynomial
                            lagvalm(isnan(lagvalm))=1; %remove singularities at z=zM
                            int=int+wqM(j+1)*...       %quadrature increment
                                lagvalm*Ck(s+tn,theta,d,par);
                        end                            %
                        INT=INT+...
                            (tau(k+1)-tau(k))/2*int;   %add to piecewise quadrature
                    end                                %
                    if ind<p                           %
                        int=0;                         %initialize quadrature on
                                                       %[-tnq,-tau_{ind}]
                        Ci=C{ind+1};                   %distributed delay kernel
                        for j=0:M                      %quadrature addends
                            theta=-((tnq-tau(ind+1))*zM(j+1)+...
                                tau(ind+1)+tnq)/2;     %point in [-tnq,-tau_{ind}]
                            if  qn-1>=q                %point in [-1,1]
                                z=2*(tn+theta)/h+...   %
                                    2*q-1;             %
                            elseif n<=Nhat             %
                                z=(2*(tn+theta)+...    %
                                    TAU+(q-1)*h)/...   %
                                    (-(q-1)*h+TAU);    %
                            else z=2*(tn+theta)/h+...  %
                                    2*Q-3;             %
                            end                        %
                            lagvalm=prod(z-zM)*...     %1st barycentric formula [1]
                                wbM(1)/(z-zM(1));      %for the 1st Lagrange polynomial
                            lagvalm(isnan(lagvalm))=1; %remove singularities at z=zM
                            int=int+wqM(j+1)*...       %quadrature increment
                                lagvalm*Ci(s+tn,theta,d,par);
                        end                            %
                        INT=INT+...
                            (tnq-tau(ind+1))/2*int;    %add to piecewise quadrature
                    end                                %
                end                                    %
                UMN1(d*(n-1)+1:d*n,(q-1)*M*d+1:((q-1)*M+1)*d)=...
                        UMN1(d*(n-1)+1:d*n,(q-1)*M*d+1:((q-1)*M+1)*d)+...
                        INT;                           %update
            end                                        %
        end                                            %
        for q=1:qn                                  %along block columns
            if qn-1>=q                              %definition of tnq
                tnq=tn+q*h;                         %
            else tnq=TAU;                           %
            end                                     %
            tnq1=tn+(q-1)*h;                        %definition of tnq1
            ind=find(tnq>=tau,1,'last')-1;          %ind=max k s.t. tau_{k}<=tnq<tau_{k+1}
            ind1=find(tnq1>=tau,1,'last')-1;        %ind1=max k s.t. tau_{k}<=tnq1<tau_{k+1}
            for m=1:M-1                             %along columns
                for k=ind1+1:ind                    %discrete delay terms in F_nqm
                    Bk=B{k};                        %discrete delay matrix
                    if  qn-1>=q                     %point in [-1,1]
                        z=2*(tn-tau(k+1))/h+2*q-1;  %
                    elseif n<=Nhat                  %
                        z=(2*(tn-tau(k+1))+...      %
                            TAU+(q-1)*h)/(-(q-1)*h+TAU);
                    else z=2*(tn-tau(k+1))/h+2*Q-3; %
                    end                             %
                    lagvalm=prod(z-zM)*...          %1st barycentric formula [1]
                        wbM(m+1)/(z-zM(m+1));       %for the mth Lagrange polynomial
                    lagvalm(isnan(lagvalm))=1;      %remove singularities at z=zM
                    UMN1(d*(n-1)+1:d*n,((q-1)*M+m)*d+1:((q-1)*M+m+1)*d)=...
                        UMN1(d*(n-1)+1:d*n,((q-1)*M+m)*d+1:((q-1)*M+m+1)*d)+...
                        lagvalm*Bk(s+tn,d,par);
                end                                 %
                if flagC==1                              %distributed delays
                    INT=0;                               %initialize piecewise
                                                         %quadrature for
                                                         %distributed delay
                                                         %terms in F_nqm
                    if ind==ind1                         %
                        int=0;                           %initialize quadrature on
                                                         %[-tnq,-tnq1]
                        Ci=C{ind+1};                     %distributed delay kernel
                        for j=0:M                        %quadrature addends
                            theta=-((tnq-tnq1)*...       %
                                zM(j+1)+tnq1+tnq)/2;     %point in [-tnq,-tnq1]
                            if  qn-1>=q                  %point in [-1,1]
                                z=2*(tn+theta)/h+2*q-1;  %
                            elseif n<=Nhat               %
                                z=(2*(tn+theta)+...      %
                                    TAU+(q-1)*h)/...     %
                                    (-(q-1)*h+TAU);      %
                            else z=2*(tn+theta)/h+2*Q-3; %
                            end                          %
                            lagvalm=prod(z-zM)*...       %1st barycentric formula [1]
                                wbM(m+1)/(z-zM(m+1));    %for the mth Lagrange polynomial
                            lagvalm(isnan(lagvalm))=1;   %remove singularities at z=zM
                            int=int+wqM(j+1)*...         %quadrature increment
                                lagvalm*Ci(s+tn,theta,d,par);
                        end                              %
                        INT=INT+(tnq-tnq1)/2*int;        %add to piecewise quadrature
                    else                                 %
                        int=0;                           %initialize quadrature on
                                                         %[-tau_{ind1+1},-tnq1]
                        Ci1=C{ind1+1};                   %distributed delay kernel
                        for j=0:M                        %quadrature addends
                            theta=-((tau(ind1+2)-...     %point in [-tau_{ind1+1},-tnq1]
                                tnq1)*zM(j+1)+tnq1+tau(ind1+2))/2;
                            if  qn-1>=q                  %point in [-1,1]
                                z=2*(tn+theta)/h+2*q-1;  %
                            elseif n<=Nhat               %
                                z=(2*(tn+theta)+TAU+(q-1)*h)/...
                                    (-(q-1)*h+TAU);      %
                            else z=2*(tn+theta)/h+2*Q-3; %
                            end                          %
                            lagvalm=prod(z-zM)*...       %1st barycentric formula [1]
                                wbM(m+1)/(z-zM(m+1));    %for the mth Lagrange polynomial
                            lagvalm(isnan(lagvalm))=1;   %remove singularities at z=zM
                            int=int+wqM(j+1)*...         %quadrature increment
                                lagvalm*Ci1(s+tn,theta,d,par);
                        end                              %
                        INT=INT+...
                            (tau(ind1+2)-tnq1)/2*int;    %add to piecewise quadrature
                        for k=ind1+2:ind                     %
                            int=0;                           %initialize quadrature on
                                                             %[-tau_{k-1},-tau_{k}]
                            Ck=C{k};                         %distributed delay kernel
                            for j=0:M                        %quadrature addends
                                theta=-((tau(k+1)-...        %point in [-tau_{k-1},-tau_{k}]
                                    tau(k))*zM(j+1)+tau(k+1)+tau(k))/2;
                                if  qn-1>=q                  %point in [-1,1]
                                    z=2*(tn+theta)/h+2*q-1;  %
                                elseif n<=Nhat               %
                                    z=(2*(tn+theta)+TAU+(q-1)*h)/...%
                                        (-(q-1)*h+TAU);      %
                                else z=2*(tn+theta)/h+2*Q-3; %
                                end                          %
                                lagvalm=prod(z-zM)*...       %1st barycentric formula [1]
                                    wbM(m+1)/(z-zM(m+1));    %for the mth Lagrange polynomial
                                lagvalm(isnan(lagvalm))=1;   %remove singularities at z=zM
                                int=int+wqM(j+1)*...         %quadrature increment
                                    lagvalm*Ck(s+tn,theta,d,par);
                            end                              %
                            INT=INT+(tau(k+1)-tau(k))/2*int; %add to piecewise quadrature
                        end                                  %
                        if ind<p                             %
                            int=0;                           %initialize quadrature on
                                                             %[-tnq,-tau_{ind}]
                            Ci=C{ind+1};                     %distributed delay kernel
                            for j=0:M                        %quadrature addends
                                theta=-((tnq-tau(ind+1))*... %point in [-tnq,-tau_{ind}]
                                    zM(j+1)+tau(ind+1)+tnq)/2; 
                                if  qn-1>=q                  %point in [-1,1]
                                    z=2*(tn+theta)/h+2*q-1;  %
                                elseif n<=Nhat               %
                                    z=(2*(tn+theta)+TAU+(q-1)*h)/...
                                        (-(q-1)*h+TAU);      %
                                else z=2*(tn+theta)/h+2*Q-3; %
                                end                          %
                                lagvalm=prod(z-zM)*...       %1st barycentric formula [1]
                                    wbM(m+1)/(z-zM(m+1));    %for the mth Lagrange polynomial
                                lagvalm(isnan(lagvalm))=1;   %remove singularities at z=zM
                                int=int+wqM(j+1)*...         %quadrature increment
                                    lagvalm*Ci(s+tn,theta,d,par);
                            end                              %
                            INT=INT+(tnq-tau(ind+1))/2*int;  %add to piecewise quadrature
                        end                                  %
                    end                                      %
                    UMN1(d*(n-1)+1:d*n,((q-1)*M+m)*d+1:((q-1)*M+m+1)*d)=...
                            UMN1(d*(n-1)+1:d*n,((q-1)*M+m)*d+1:((q-1)*M+m+1)*d)+...
                            INT;                             %update
                end                                          %
            end                                              %
        end                                                  %
    end                                                      %
    for n=1:Nhat                               %along rows for n=1,...,Nhat
        tn=OmegaN(n);                          %current point
        tnQ1=tn+(Q-1)*h;                       %definition of tnQ1
        ind1=find(tnQ1>=tau,1,'last')-1;       %ind=max k s.t. tau_{k}<=tnQ1<tau_{k+1}
        for k=ind1+1:p                         %discrete delay terms in F_nQM
            Bk=B{k};                           %discrete delay matrix
            z=(2*(tn-tau(k+1))+TAU+...
                (Q-1)*h)/(TAU-(Q-1)*h);        %point in [-1,1]
            lagvalM=prod(z-zM)*...             %1st barycentric formula [1]
                wbM(M+1)/(z-zM(M+1));          %for the mth Lagrange polynomial
            lagvalM(isnan(lagvalM))=1;         %remove singularities at z=zM
            UMN1(d*(n-1)+1:d*n,Q*M*d+1:(Q*M+1)*d)=...
                UMN1(d*(n-1)+1:d*n,Q*M*d+1:(Q*M+1)*d)+...
                lagvalM*Bk(s+tn,d,par);
        end                                    %
        if flagC==1                            %distributed delays
            INT=0;                             %initialize piecewise quadrature
                                               %for distributed delay terms in F_nQM
            int=0;                             %initialize quadrature on
                                               %[-tau_{ind1+1},-tnQ1]
            Ci1=C{ind1+1};                     %distributed delay kernel
            for j=0:M                          %quadrature addends
                theta=-((tau(ind1+2)-...       %point in [-tau_{ind1+1},-tnQ1]
                    tnQ1)*zM(j+1)+tnQ1+tau(ind1+2))/2;     
                z=(2*(tn+theta)+TAU+...        %point in [-1,1]
                    (Q-1)*h)/(TAU-(Q-1)*h); 
                lagvalm=prod(z-zM)*...         %1st barycentric formula [1]
                    wbM(M+1)/(z-zM(M+1));      %for the Mth Lagrange polynomial
                lagvalm(isnan(lagvalm))=1;     %remove singularities at z=zM
                int=int+wqM(j+1)*...           %quadrature increment
                    lagvalm*Ci1(s+tn,theta,d,par);
            end                                %
            INT=INT+(tau(ind1+2)-tnQ1)/2*int;  %add to piecewise quadrature
            for k=ind1+2:p                     %
                int=0;                         %initialize quadrature on
                                               %[-tau_{k+1},-tau_{k}]
                Ck=C{k};                       %distributed delay kernel
                for j=0:M                      %quadrature addends
                    theta=-((tau(k+1)-...      %point in [-tau_{k+1},-tau_{k}]
                        tau(k))*zM(j+1)+tau(k+1)+tau(k))/2;
                    z=(2*(tn+theta)+TAU+(Q-1)*h)/(TAU-(Q-1)*h); %point in [-1,1]
                    lagvalm=prod(z-zM)*...     %1st barycentric formula [1]
                        wbM(M+1)/(z-zM(M+1));  %for the Mth Lagrange polynomial
                    lagvalm(isnan(lagvalm))=1; %remove singularities at z=zM
                    int=int+wqM(j+1)*...       %quadrature increment
                        lagvalm*Ck(s+tn,theta,d,par);
                end                            %
                INT=INT+...
                    (tau(k+1)-tau(k))/2*int;   %add to piecewise quadrature
            end                                %
            UMN1(d*(n-1)+1:d*n,Q*M*d+1:(Q*M+1)*d)=...
                UMN1(d*(n-1)+1:d*n,Q*M*d+1:(Q*M+1)*d)+...
                INT;                           %update
        end                                    %
    end                                        %
    for n=Nhat+1:N                             %along rows for n=Nhat+1,...,N
        tn=OmegaN(n);                          %current point
        tnQ2=tn+(Q-2)*h;                       %defintion of tnQ2
        ind2=find(tnQ2>=tau,1,'last')-1;       %ind=max k s.t. tau_{k}<=tnQ2<tau_{k+1}
        for k=ind2+1:p                         %discrete delay terms in F_nQ0
            Bk=B{k};                           %discrete delay matrix
            z=2*(tn-tau(k+1))/h+2*Q-3;         %point in [-1,1]
            lagvalM=prod(z-zM)*...             %1st barycentric formula [1]
                wbM(M+1)/(z-zM(M+1));          %for the mth Lagrange polynomial
            lagvalM(isnan(lagvalM))=1;         %remove singularities at z=zM
            UMN1(d*(n-1)+1:d*n,(Q-1)*M*d+1:((Q-1)*M+1)*d)=...
                UMN1(d*(n-1)+1:d*n,(Q-1)*M*d+1:((Q-1)*M+1)*d)+...
                lagvalM*Bk(s+tn,d,par);
        end                                    %
        if flagC==1                            %distributed delays
            INT=0;                             %initialize piecewise quadrature
                                               %for distributed delay terms in F_nQ0
            int=0;                             %initialize quadrature on
                                               %[-tau_{ind2+1},-tnQ2]
            Ci2=C{ind2+1};                     %distributed delay kernel
            for j=0:M                          %quadrature addends
                theta=-((tau(ind2+2)-...       %point in [-tau_{ind2+1},-tnQ2]
                    tnQ2)*zM(j+1)+tnQ2+tau(ind2+2))/2;     
                z=2*(tn+theta)/h+2*Q-3;        %point in [-1,1]
                lagvalm=prod(z-zM)*...         %1st barycentric formula [1]
                    wbM(M+1)/(z-zM(M+1));      %for the Mth Lagrange polynomial
                lagvalm(isnan(lagvalm))=1;     %remove singularities at z=zM
                int=int+wqM(j+1)*...           %quadrature increment
                    lagvalm*Ci2(s+tn,theta,d,par);
            end                                %
            INT=INT+(tau(ind2+2)-tnQ2)/2*int;  %add to piecewise quadrature
            for k=ind2+2:p                     %
                int=0;                         %initialize quadrature on
                                               %[-tau_{k-1},-tau_{k}]
                Ck=C{k};                       %distributed delay kernel
                for j=0:M                      %quadrature addends
                    theta=-((tau(k+1)-tau(k))*zM(j+1)+...
                        tau(k+1)+tau(k))/2;    %point in [-tau_{k-1},-tau_{k}]
                    z=2*(tn+theta)/h+2*Q-3;    %point in [-1,1]
                    lagvalm=prod(z-zM)*...     %1st barycentric formula [1]
                        wbM(M+1)/(z-zM(M+1));  %for the Mth Lagrange polynomial
                    lagvalm(isnan(lagvalm))=1; %remove singularities at z=zM
                    int=int+wqM(j+1)*...       %quadrature increment
                        lagvalm*Ck(s+tn,theta,d,par);
                end                            %
                INT=INT+...
                    (tau(k+1)-tau(k))/2*int;   %add to piecewise quadrature
            end                                %
            UMN1(d*(n-1)+1:d*n,(Q-1)*M*d+1:((Q-1)*M+1)*d)=...
                UMN1(d*(n-1)+1:d*n,(Q-1)*M*d+1:((Q-1)*M+1)*d)+...
                INT;                           %update
        end                                    %
    end                                        %
end                                            %

%% MATRIX UN2
UN2=zeros(d*N);                            %initialization
for n=1:N                                  %along rows
    tn=OmegaN(n);                          %current point
    mTAUtn=min(TAU,tn);                    %
    ind=find(mTAUtn>=tau,1,'last')-1;      %ind=max k s.t. tau_{k}<=mTAUtn<tau_{k+1}
    for i=1:N                              %along columns
        t=tn-tau(1);                       %inner integral upper extremum
        int=0;                             %initialize inner quadrature sum
                                           %for current time term
        for j=0:M                          %inner quadrature addends
            z=1-t*(zM(j+1)+1)/h;           %point in [-1,1]
            lagvaln=prod(z-zN)*...         %1st barycentric formula [1]
                wbN(i)/(z-zN(i));          %for the ith Lagrange polynomial
            lagvaln(isnan(lagvaln))=1;     %remove singularities at z=zN
            int=int+wqM(j+1)*lagvaln;      %inner quadrature increment
        end                                %
        UN2(1+d*(n-1):d*n,1+d*(i-1):d*i)=UN2(1+d*(n-1):d*n,1+d*(i-1):d*i)+...
            t/2*int*A(s+tn,d,par);         %final inner quadrature
        for k=1:ind                        %discrete delay terms
            Bk=B{k};                       %discrete delay matrix
            t=tn-tau(k+1);                 %inner integral upper extremum
            int=0;                         %initialize inner quadrature sum
                                           %for discrete delay terms
            for j=0:M                      %inner quadrature addends
                z=1-t*(zM(j+1)+1)/h;       %point in [-1,1]
                lagvaln=prod(z-zN)*...     %1st barycentric formula [1]
                    wbN(i)/(z-zN(i));      %for the ith Lagrange polynomial
                lagvaln(isnan(lagvaln))=1; %remove singularities at z=zN
                int=int+wqM(j+1)*lagvaln;  %inner quadrature increment
            end                            %
            UN2(1+d*(n-1):d*n,1+d*(i-1):d*i)=...
                UN2(1+d*(n-1):d*n,1+d*(i-1):d*i)+t/2*int*...
                feval(Bk,s+tn,d,par);      %final inner quadrature
        end                                %
        if flagC==1                                %distributed delays
            INT=0;                                 %initialize piecewise quadrature
                                                   %for distributed delay terms
            for k=1:ind                            %
                int=0;                             %initialize quadrature on
                                                   %[-tau_{k-1},-tau_{k}]
                Ck=C{k};                           %distributed delay kernel
                for j=0:M                          %quadrature addends
                    theta=-((tau(k+1)-tau(k))*...  %point in [-tau_{k-1},-tau_{k}]
                        zM(j+1)+tau(k+1)+tau(k))/2;  
                    t=tn+theta;                    %integral upper extremum
                    intlag=0;                      %initialize quadrature sum
                                                   %for inner integrals
                    for l=0:M                      %inner quadrature addends
                        z=1-t*(zM(l+1)+1)/h;       %point in [-1,1]
                        lagvaln=prod(z-zN)*...     %1st barycentric formula [1]
                            wbN(i)/(z-zN(i));      %for the ith Lagrange polynomial
                        lagvaln(isnan(lagvaln))=1; %remove singularities at z=zN
                        intlag=intlag+...          %inner quadrature increment
                            wqM(l+1)*lagvaln;
                    end                            %
                    int=int+...                    %quadrature increment
                        wqM(j+1)*t/2*intlag*Ck(s+tn,theta,d,par);
                end                                %
                INT=INT+(tau(k+1)-tau(k))/2*int;   %add to piecewise quadrature
            end                                    %
            if ind<p                               %
                int=0;                             %initialize quadrature on
                                                   %[-mTAUtn,-tau_{ind}]
                Ci=C{ind+1};                       %distributed delay kernel
                for j=0:M                          %quadrature addends
                    theta=-((mTAUtn-...            %point in [-mTAUtn,-tau_{ind}]
                        tau(ind+1))*zM(j+1)+mTAUtn+tau(ind+1))/2;
                    t=tn+theta;                    %integral upper extremum
                    intlag=0;                      %initialize quadrature sum
                                                   %for inner integrals
                    for l=0:M                      %inner quadrature addends
                        z=1-t*(zM(l+1)+1)/h;       %point in [-1,1]
                        lagvaln=prod(z-zN)*...     %1st barycentric formula [1]
                            wbN(i)/(z-zN(i));      %for the ith Lagrange polynomial
                        lagvaln(isnan(lagvaln))=1; %remove singularities at z=zN
                        intlag=intlag+...          %inner quadrature increment
                            wqM(l+1)*lagvaln;
                    end                            %
                    int=int+...                    %quadrature increment
                        wqM(j+1)*t/2*intlag*Ci(s+tn,theta,d,par);
                end                                %
                INT=INT+(mTAUtn-tau(ind+1))/2*int; %add to piecewise quadrature
            end                                    %
            UN2(1+d*(n-1):d*n,1+d*(i-1):d*i)=...
                    UN2(1+d*(n-1):d*n,1+d*(i-1):d*i)+...
                    INT;                           %update
        end                                        %
    end                                            %
end                                                %

%% MATRIX TMN
TMN=TM1+TMN2*((eye(d*N)-UN2)\UMN1); %evolution operator final discretization

%% COMPUTATION OF EIGENVALUES
mu=eig(TMN);           %approximated eigenvalues
%ordered by decreasing modulus
%work around missing 'ComparisonMethod' option of sort in Octave
if isreal(mu)
    [mu,ii]=sort([1i;mu],'descend');
    mu(find(ii==1))=[];
else
    mu=sort(mu,'descend');
end

%% AUXILIARY FUNCTIONS
function [d,tau,p,A,B,C,flagC]=conversion(myDDE,par)
%CONVERSION convert DDE models.
%  [d,tau,p,A,B,C,flagC]=CONVERSION(myDDE) converts the original system of
%  d linear DDEs
%
%  x'(t)=Atilde(t)x(t)
%   +sum_{u=1}^{q}Btilde_{u}(t)*x(t-d_{u})
%   +sum_{v=1}^{w}int_{-l_{v}}^{-r_{v}}Ctilde_{v}(t,theta)x(t+theta)dtheta
%
%  contained in the script "myDDE.m" into
%
%  x'(t)=A(t)x(t)
%   +sum_{k=p}^{p}B_{k}(t)*x(t-tau_{k})
%   +sum_{k=1}^{p}int_{-tau_{k}}^{-tau_{k-1}}C_{k}(t,theta)x(t+theta)dtheta
%
%  with ordered delays according to the notation in [1, Chapter 7].
%
%  Version 1.0, june 5, 2014.

%% DDE FUNCTIONS AND PARAMETERS
eval(myDDE); %load functions and parameters of the original DDE contained
             %in the script "myDDE.m".

%% DELAYS
q=length(dd); %number of original discrete delay terms
w=length(l);  %number of original distributed delay terms
if w>0                           %eliminate distributed terms with
    ind=find(l-r~=0);            %null integration interval
    w=length(ind);               %
    Cbar=cell(w,1);              %
    for v=1:w                    %
        Cbar{v}=Ctilde{ind(v)};  %
        l(v)=l(ind(v));          %
        r(v)=r(ind(v));          %
    end                          %
    l=l(1:w);                    %
    r=r(1:w);                    %
    Ctilde=Cbar;                 %
end
tau=unique(sort([0,dd,l,r])); %vector of ordered delays
p=length(tau)-1;              %[tau_{0},tau_{1},...,tau_{p}]

%% CURRENT TIME AND DISCRETE DELAY TERMS
Bbar=cell(p+1,1);
Bbar{1}=@(t,d,par) Atilde(t,d,par);
for k=2:p+1
    Bbar{k}=@(t,d,par) zeros(d);
end
for u=1:q
    k=find(tau==dd(u));
    Bbark=Bbar{k};
    Btildeu=Btilde{u};
    Bbar{k}=@(t,d,par)Bbark(t,d,par)+Btildeu(t,d,par);
end
A=Bbar{1};
B=cell(p,1);
for k=1:p
    B{k}=Bbar{k+1};
end

%% DISTRIBUTED DELAY TERMS
C=cell(p,1);
if isempty(l)
    flagC=0;
else flagC=1;
    for k=1:p
        C{k}=@(t,theta,d,par) zeros(d);
    end
    LR=[l;r];
    [LR,ind]=sort(LR,'descend');
    l=LR(1,:);
    r=LR(2,:);
    ind=find(ind(1,:)==2);
    for v=1:length(ind)
        Ctildev=Ctilde{ind(v)};
        Ctilde{ind(v)}=@(t,theta,d,par)-Ctildev(t,theta,d,par);
    end
    for v=1:w
        Ctildev=Ctilde{v};
        ind=find((-l(v)<=-tau)&(-tau<-r(v)));
        for k=1:length(ind)
            Ck=C{ind(k)-1};
            C{ind(k)-1}=@(t,theta,d,par)Ck(t,theta,d,par)+...
                Ctildev(t,theta,d,par);
        end
    end
end

function wb=barywei(x)
%WEIGHTS barycentric weights.
%  wb=BARYWEI(x) returns the weights wb for Lagrange barycentric
%  interpolation for the nodes x according to [1].
%
%  Original version from [1].

N=length(x)-1;
dx=ones(N+1,1);
wb=dx;
for m=2:N+1
    for i=1:m-1
        dx(i)=x(m)-x(i);
        wb(i)=-dx(i)*wb(i);
    end
    wb(m)=prod(dx(1:m));
end
wb=1./wb';

function wq=quadwei(N)
%QUADWEI Clenshaw-Curtis quadrature weights.
%  wq=QUADWEI(N) returns the weights wq of the Clenshaw-Curtis quadrature,
%  i.e., the one based on Chebyshew II nodes in [-1,1], see [4].
%
%  Original version from [4].

if N==0
    wq=2;
else p=pi*(0:N)'/N;
    wq=zeros(1,N+1);
    ii=2:N;
    v=ones(N-1,1);
    if mod(N,2)==0
        wq(1)=1/(N^2-1);
        wq(N+1)=wq(1);
        for k=1:N/2-1
            v=v-2*cos(2*k*p(ii))/(4*k^2-1);
        end
        v=v-cos(N*p(ii))/(N^2-1);
    else
        wq(1)=1/N^2;
        wq(N+1)=wq(1);
        for k=1:(N-1)/2
            v=v-2*cos(2*k*p(ii))/(4*k^2-1);
        end
    end
    wq(ii)=2*v/N;
end