% SCRIPT "myDDE_ppBDA.m" relevant to [1, Section 8.2]
%
% This script defines functions and parameters of the system of d linear
% Delay Differential Equations (DDEs)
%
% x'(t)=Atilde(t)x(t)
%   +sum_{u=1}^{q}Btilde_{u}(t)*x(t-d_{u})
%   +sum_{v=1}^{w}int_{-l_{v}}^{-r_{v}}Ctilde_{v}(t,theta)x(t+theta)dtheta
%
% according to the notation in [1, Chapter 7].
%
% The script can be used in
% -"eigAM.m" to approximate the eigenvalues of the infinitesimal generator
%  in the AUTONOMOUS case based on the method described in [1, Chapter 5]
%  and originally in [3].
% -"eigTMN.m" to approximate the eigenvalues of the evolution operator in
%  the NONAUTONOMOUS case based on the method described in
%  [1, Chapter 6] and originally in [2].
%
% Version 1.0, june 5, 2014. 
%
% References:
%  [1] D. Breda, S. Maset and R. Vermiglio, "Stability of linear delay
%      differential equations - A numerical approach with MATLAB", in
%      Control, Automation and Robotics, T. Basar, A. Bicchi and M. Krstic
%      eds., Springer, New York, 2015, ISBN 978-1-4939-2106-5.
%  [2] D. Breda, S. Maset and R. Vermiglio, "Approximation of eigenvalues
%      of evolution operators for linear retarded functional differential
%      equations", SIAM J. Numer. Anal. 50(3):1456-1483, 2012.
%  [3] D. Breda, S. Maset and R. Vermiglio, "Pseudospectral differencing
%      methods for characteristic roots of delay differential equations",
%      SIAM J. Sci. Comput. 27(2):482-495, 2005.

%% MEMO LIST OF POSSIBLE PARAMETERS
%par(1)=r, preys growth rate
%par(2)=K, preys carrying capacity
%par(3)=b, predators capture rate
%par(4)=k1, predators handling time
%par(5)=k2, predators interference
%par(6)=n, predators birth rate
%par(7)=m, adult predators death rate
%par(8)=mj, juvenile predators death rate
%par(9)=s, predators maturation delay
%par(10=s1, predators gestation delay
%par(11)=xbar1, preys at equilibrium
%par(12)=xbar2, adult predators at equilibrium

%% DIMENSION OF THE DDE
d=2;

%% CURRENT TIME TERM
Atilde=@(t,d,par) [par(1)*(1-2*par(11)/par(2))-...
    par(3)*par(12)*(1+par(5)*par(12))/...
    ((1+par(4)*par(11)+par(5)*par(12))^2),...
    -par(3)*par(11)*(1+par(4)*par(11))/...
    ((1+par(4)*par(11)+par(5)*par(12))^2);...
    0,-par(7)];

%% DISCRETE DELAY TERMS
dd=[par(9),par(9)+par(10)];
Btilde{1}=@(t,d,par) [0,0;0,par(6)*exp(-par(8)*par(9))*...
    par(3)*par(11)/(1+par(4)*par(11)+par(5)*par(12))];
Btilde{2}=@(t,d,par) [0,0;par(6)*exp(-par(8)*par(9))*...
    par(3)*par(12)*(1+par(5)*par(12))/...
    ((1+par(4)*par(11)+par(5)*par(12))^2),...
    -par(6)*exp(-par(8)*par(9))*par(5)*...
    par(3)*par(11)*par(12)/...
    ((1+par(4)*par(11)+par(5)*par(12))^2)];

%% DISTRIBUTED DELAY TERMS
l=[];
r=[];
Ctilde{1}=@(t,theta,d,par) [];